#!/usr/bin/env python3
#
# Copyright 2017 Ettus Research, a National Instruments Company
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Aurora BIST command line utility
"""

from __future__ import print_function
import argparse
import usrp_mpm as mpm
from usrp_mpm.sys_utils.uio import UIO
from usrp_mpm.aurora_control import AuroraControl

########################################################################
# command line options
########################################################################
def parse_args():
    " Create argparser, return args "
    parser = argparse.ArgumentParser(
        description='Controller for Ettus Aurora BIST Engine'
    )
    parser.add_argument(
        '--uio-dev', default='misc-auro-regs0',
        help='UIO device for master device peeks and pokes'
    )
    parser.add_argument(
        '--base-addr', type=int, default=0,
        help='Base address for register read/writes'
    )
    parser.add_argument(
        '--slave-uio-dev', default=None,
        help='UIO device for slave device peeks and pokes'
    )
    parser.add_argument(
        '--slave-base-addr', type=int, default=0,
        help='Base address for register read/writes on slave'
    )
    parser.add_argument(
        '--test', default='ber', choices=['ber', 'latency'],
        help='Type of test to run'
    )
    parser.add_argument(
        '--duration', type=int, default=10, help='Duration of test in seconds'
    )
    parser.add_argument(
        '--rate', type=int, default=1245, help='BIST throughput in MB/s'
    )
    parser.add_argument(
        '--loopback', action="store_true",
        help="Don't run a test, but set this Aurora into loopback mode"
    )
    return parser.parse_args()

########################################################################
# main
########################################################################
def main():
    " Dawaj, dawaj! "
    args = parse_args()
    # Initialize logger for downstream components
    mpm.get_main_logger().getChild('main')
    master_core_uio = UIO(label=args.uio_dev, read_only=False)
    master_core_uio.open()
    if args.slave_uio_dev:
        slave_core_uio = UIO(label=args.uio_dev, read_only=False)
        slave_core_uio.open()
    try:
        master_core = AuroraControl(master_core_uio, args.base_addr)
        slave_core = None if args.slave_uio_dev is None else AuroraControl(
            slave_core_uio,
            args.slave_base_addr,
        )
        if args.loopback:
            master_core.reset_core()
            master_core.set_loopback(enable=True)
            return True
        # Run BIST
        if args.test == 'ber':
            print("Performing BER BIST test.")
            master_core.run_ber_loopback_bist(
                args.duration,
                args.rate * 8e6,
                slave_core,
            )
        else:
            print("Performing Latency BIST test.")
            master_core.run_latency_loopback_bist(
                args.duration,
                args.rate * 8e6,
                slave_core,
            )
    except Exception as ex:
        print("Unexpected exception: {}".format(str(ex)))
        return False
    finally:
        master_core_uio.close()
        if args.slave_uio_dev:
            slave_core_uio.close()

if __name__ == '__main__':
    exit(not main())



#
# Copyright 2020 Ettus Research, a National Instruments Brand
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
RPC/MPM utilities for debugging USRPs
"""

from enum import Enum
import multiprocessing
import queue
import signal
from mprpc import RPCClient
from mprpc.exceptions import RPCError

MPM_RPC_PORT = 49601

def _claim_loop(client, cmd_q, token_q):
    """
    Process that runs a claim loop.

    This function should be run in its own thread. Communication to the outside
    thread happens with two queues: A command queue, and a token queue. The
    command queue is used to pass in one of these commands: claim, unclaim, or
    exit.
    The token queue is used to read back the current token.
    """

    def _sig_term_handler(_signo, _stack):
        """
        gracefully terminate claim loop
        """
        cmd_q.put("exit")

    command = None
    token = None
    exit_loop = False

    signal.signal(signal.SIGTERM, _sig_term_handler)

    try:
        while not exit_loop:
            try:
                command = cmd_q.get(True, 1)
            except queue.Empty:
                # if we do not receive a command, reclaim device
                if token:
                    client.call('reclaim', token)
                continue

            if command == 'claim':
                if not token:
                    try:
                        token = client.call('claim', 'UHD')
                    except RPCError as ex:
                        # catch RPC errors here so the loop keeps running
                        # and token queue receives an (empty) value
                        print(str(ex))
                else:
                    print("Already have claim")
                token_q.put(token)
            elif command == 'unclaim':
                if token:
                    client.call('unclaim', token)
                token = None
                token_q.put(None)
            elif command == 'exit':
                if token:
                    client.call('unclaim', token)
                token = None
                token_q.put(None)
                exit_loop = True
    except RPCError as ex:
        print("Unexpected RPC error in claimer loop!")
        print(str(ex))

class MPMClaimer:
    """
    Holds a claim.
    """
    def __init__(self, host, port):
        self.token = None
        self._cmd_q = multiprocessing.Queue()
        self._token_q = multiprocessing.Queue()
        client = RPCClient(host, port, pack_params={'use_bin_type': True})
        self._claim_loop = multiprocessing.Process(
            target=_claim_loop,
            name="Claimer Loop",
            args=(client, self._cmd_q, self._token_q)
        )
        self._claim_loop.daemon = True
        self._claim_loop.start()

    def exit(self):
        """
        Unclaim device and exit claim loop.
        """
        self.unclaim()
        self._cmd_q.put('exit')
        self._claim_loop.join()

    def unclaim(self):
        """
        Unclaim device.
        """
        self._cmd_q.put('unclaim')
        self.token = self._token_q.get(True, 1)

    def claim(self):
        """
        Claim device.
        """
        self._cmd_q.put('claim')
        self.token = self._token_q.get(True, 1)
        if not self.token:
            raise RuntimeError("Failed to claim device")

    def get_token(self):
        """
        Get current token (if any)
        """
        if not self._token_q.empty():
            self.token = self._token_q.get(False)
        return self.token

class InitMode(Enum):
    """
    Init modes for MPM session
    """
    Hijack = 1
    Claim = 2
    Noclaim = 3

# Ironically, this class will have too many public methods, Pylint just doesn't
# know it yet.
# pylint: disable=too-few-public-methods
class MPMClient:
    """
    MPM RPC Client: Will make all MPM commands accessible as Python methods.
    """
    def __init__(self, init_mode, host, port=MPM_RPC_PORT, token=None):
        assert isinstance(init_mode, InitMode)
        print("[MPMRPC] Attempting to connect to {host}:{port}...".format(
            host=host, port=port
        ))
        self._remote_methods = []
        if init_mode == InitMode.Hijack:
            assert token
            self._token = token
        if init_mode == InitMode.Claim:
            self._claimer = MPMClaimer(host, port)
            self._claimer.claim()
            self._token = self._claimer.token

        try:
            self._client = RPCClient(host, port, pack_params={'use_bin_type': True})
            print("[MPMRPC] Connection successful.")
        except Exception as ex:
            print("[MPMRPC] Connection refused: {}".format(ex))
            raise RuntimeError("RPC connection refused.")
        print("[MPMRPC] Getting methods...")
        methods = self._client.call('list_methods')
        for method in methods:
            self._add_command(*method)
        print("[MPMRPC] Added {} methods.".format(len(methods)))

    def __del__(self):
        if hasattr(self, '_claimer'):
            self._claimer.exit()

    def claim(self):
        """
        Use claimer (instead of RPC method) to claim MPM device
        """
        self._claimer.claim()

    def unclaim(self):
        """
        Use claimer (instead of RPC method) to unclaim MPM device
        """
        self._claimer.unclaim()

    def exit(self):
        """
        Use claimer (instead of RPC method) to unclaim MPM device and exit claim loop
        """
        self._claimer.exit()

    def _add_command(self, command, docs, requires_token):
        """
        Add a command to the current session
        """
        if not hasattr(self, command):
            new_command = lambda *args, **kwargs: self._rpc_template(
                str(command), requires_token, *args, **kwargs
            )
            new_command.__doc__ = docs
            setattr(self, command, new_command)
            self._remote_methods.append(command)

    def _rpc_template(self, command, requires_token, *args, **kwargs):
        """
        Template function to create new RPC shell commands
        """
        if requires_token and not self._token:
            raise RuntimeError(
                "[MPMRPC] Cannot execute `{}' -- no claim available!"
                .format(command))
        # Put token as the first argument if required:
        if requires_token:
            args = (self._token,) + args
        if kwargs:
            return self._client.call(command, *args, **kwargs)
        if args:
            return self._client.call(command, *args)
        return self._client.call(command)
# pylint: enable=too-few-public-methods



#
# Copyright 2020 Ettus Research, a National Instruments Brand
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
RFSA/RFSG-type measurement devices
"""
import os
import platform
if platform.system() == 'Windows':
    from ctypes import WinError, CDLL, byref, POINTER
    from ctypes import c_bool, c_int, c_int32, c_double, c_char_p

# pylint: disable=unused-argument
def _check(result, func, arguments):
    """
    Test result for error. Throw WinError, if result is not 0 using result as
    error index. This is a convenient default implementation for WIN API calls
    """
    assert os.name == "nt" # throwing WinError works on Windows only.
    if result != 0:
        error = WinError(result,
                         "External library call failed with {}".format(result))
        raise error

def _wrap(lib, func_name, res_type, arg_types, check_error=True):
    """
    Ease usage of API functions imported using ctypes library.
    :param lib: CDLL loaded by ctypes
    :param func_name: Function import to uses
    :param res_type: result type of call (as ctype)
    :param arg_types: argument list of call (as ctype)
    :param check_error: error checker (default uses _check)
    :return: Python function
    """
    func = lib.__getattr__(func_name)
    func.restype = res_type
    func.argtypes = arg_types
    if check_error:
        func.errcheck = _check
    return func

def get_modinst_devices(driver):
    """
    Creates a list of instruments matching the current driver. Set
    self._driver before calling this method
    :return: list of nimodinst device objects
    :throws: RuntimeError if called on non-Windows systems.
    """
    if platform.system() != 'Windows':
        raise RuntimeError('Cannot connect to ModInst '
                           'devices on non-Windows systems.')
    # make sure NI ModInst package is only loaded when required
    # pylint: disable=import-outside-toplevel
    import nimodinst
    session = nimodinst.Session(driver)
    return session.devices

def get_modinst_device(driver, name):
    """
    Get exact on device for the current driver. Use (optional) name to
    narrow the list if more than one device is installed for the current
    driver.
    :param name: name of devices to get
    :return: NI ModInst device class
    :throws: RuntimeError if no or more than one device was found.
    """
    devices = [device for device in get_modinst_devices(driver)
               if not name or name == device.device_name]
    if len(devices) > 1:
        print("Found %s devices:" % driver)
        for device in devices:
            print("* %s" % device.device_name)
        raise RuntimeError("Found more than one measurement device. "
                           "Please limit the query!")
    if not devices:
        raise RuntimeError("No measurement device found!")
    return devices[0]


class RFSADevice:
    """
    Class to measure power using devices from the NI-RFSA family. This class
    make use of the RFmx SpecAn TXP measurement. So this library must be
    installed as well in addition to the RFSA driver.
    """
    RFMXSPECAN_ATTR_REFERENCE_LEVEL = 0x00100002

    RFMXSPECAN_VAL_TXP_RBW_FILTER_TYPE_GAUSSIAN = 1
    RFMXSPECAN_VAL_TXP_AVERAGING_ENABLED_TRUE = 1
    RFMXSPECAN_VAL_TXP_AVERAGING_TYPE_RMS = 0

    def __init__(self, options):
        self.session = c_int(0)

        self._generate_specan_calls()

        device = get_modinst_device("NI-RFSA", options.get("name", None))

        self._RFmxSpecAn_Initialize(device.device_name.encode(),
                                    "".encode(), byref(self.session), 0)

    def __del__(self):
        if self.session.value > 0:
            self._RFmxSpecAn_Close(self.session, 0)

    def _generate_specan_calls(self):
        """
        Generate needed Python function in class to do power measurements.
        """
        lib = CDLL("niRFmxSpecAn.dll")
        # disable invalid name error to use function names as declared in C-DLL
        # pylint: disable=invalid-name
        self._RFmxSpecAn_Initialize = _wrap(
            lib,
            "RFmxSpecAn_Initialize",
            c_int,
            [c_char_p, c_char_p, POINTER(c_int), c_int]
        )
        self._RFmxSpecAn_Close = _wrap(
            lib,
            "RFmxSpecAn_Close",
            c_int,
            [c_int, c_int]
        )
        self._RFmxSpecAn_SetSelectedPorts = _wrap(
            lib,
            "RFmxSpecAn_SetSelectedPorts",
            c_int,
            [c_int, c_char_p, c_char_p]
        )
        self._RFmxSpecAn_CfgRF = _wrap(
            lib,
            "RFmxSpecAn_CfgRF",
            c_int,
            [c_int, c_char_p, c_double, c_double, c_double]
        )
        self._RFmxSpecAn_TXPCfgRBWFilter = _wrap(
            lib,
            "RFmxSpecAn_TXPCfgRBWFilter",
            c_int,
            [c_int, c_char_p, c_double, c_int32, c_double]
        )
        self._RFmxSpecAn_TXPCfgMeasurementInterval = _wrap(
            lib,
            "RFmxSpecAn_TXPCfgMeasurementInterval",
            c_int32,
            [c_int, c_char_p, c_double]
        )
        self._RFmxSpecAn_TXPCfgAveraging = _wrap(
            lib,
            "RFmxSpecAn_TXPCfgAveraging",
            c_int32,
            [c_int, c_char_p, c_int32, c_int32, c_int32,]
        )
        self._RFmxSpecAn_TXPRead = _wrap(
            lib,
            "RFmxSpecAn_TXPRead",
            c_int,
            [c_int, c_char_p, c_double, POINTER(c_double),
             POINTER(c_double), POINTER(c_double), POINTER(c_double)]
        )
        self._RFmxSpecAn_GetAttributeF64 = _wrap(
            lib,
            "RFmxSpecAn_GetAttributeF64",
            c_int,
            [c_int, c_char_p, c_int, POINTER(c_double)]
        )
        self._RFmxSpecAn_SetAttributeF64 = _wrap(
            lib,
            "RFmxSpecAn_SetAttributeF64",
            c_int,
            [c_int, c_char_p, c_int, c_double]
        )


    def init_power_meter(self):
        """
        Initialize power meter (derived from "RFmxSpecAn TXP(Basic)" example)
        """
        rbw = c_double(100E3)
        filt = c_int32(RFSADevice.RFMXSPECAN_VAL_TXP_RBW_FILTER_TYPE_GAUSSIAN)
        rrc_alpha = c_double(0)
        meas_int = c_double(1E-3)
        avg = c_int32(RFSADevice.RFMXSPECAN_VAL_TXP_AVERAGING_ENABLED_TRUE)
        avg_count = c_int32(10)
        avg_type = c_int32(RFSADevice.RFMXSPECAN_VAL_TXP_AVERAGING_TYPE_RMS)
        self._RFmxSpecAn_SetSelectedPorts(self.session, b"", b"")
        self._RFmxSpecAn_TXPCfgRBWFilter(self.session, b"", rbw, filt, rrc_alpha)
        self._RFmxSpecAn_TXPCfgMeasurementInterval(self.session, b"", meas_int)
        self._RFmxSpecAn_TXPCfgAveraging(self.session, b"", avg, avg_count, avg_type)

    def set_frequency(self, freq):
        """
        Set measurement frequency and reset ref level.
        """
        freq = c_double(freq)
        ref_level = c_double(-20) # start with a low ref level
        ext_att = c_double(0) # calibration script takes responsibility for
                              # external attenuation, so no need to set it here
        self._RFmxSpecAn_CfgRF(self.session, b"", freq, ref_level, ext_att)

    def _get_attribute(self, attr_id):
        """
        Wrapper to ease read RFSA double attributes.
        """
        result = c_double(0)
        self._RFmxSpecAn_GetAttributeF64(self.session, b"", attr_id, byref(result))
        return result.value

    def _set_attribute(self, attr_id, value):
        """
        Wrapper to ease write RFSA double attributes.
        """
        attr = c_double(value)
        self._RFmxSpecAn_SetAttributeF64(self.session, b"", attr_id, attr)

    def get_reference_level(self):
        """
        Return reference level used for measurements
        """
        return self._get_attribute(RFSADevice.RFMXSPECAN_ATTR_REFERENCE_LEVEL)

    def set_reference_level(self, value):
        """
        Set reference level used for measurements
        """
        self._set_attribute(RFSADevice.RFMXSPECAN_ATTR_REFERENCE_LEVEL, value)

    def get_power_dbm(self):
        """
        Measure power at current frequency. Reference level starts at -20dBm
        (see set_frequency) and is increased in 5dB steps until measurement
        succeeds. Returns the measured averaged power.
        """
        timeout = c_double(1.0)
        mean_val = c_double(0)
        peak2avg_val = c_double(0)
        min_val = c_double(0)
        max_val = c_double(0)
        while True:
            try:
                self._RFmxSpecAn_TXPRead(self.session, b"", timeout,
                                         byref(mean_val), byref(peak2avg_val),
                                         byref(min_val), byref(max_val))
                return mean_val.value
            except OSError as ex:
                # increase ref level on ADC or DSA overload
                if getattr(ex, 'winerror') in [
                    0x0005B10A, # DSP overflow (RFSA)
                    0x0005B10B, # ADC overload (RFSA)
                    0x3FFA9001, # ADC overload (mxl)
                    ]:
                    self.set_reference_level(self.get_reference_level() + 5)
                else:
                    raise ex


class RFSGDevice:
    """
    Class to generate CW tone using devices from the NI-RFSG family
    """
    NIRFSG_ATTR_POWER_LEVEL = 1250002
    NIRFSG_VAL_CW = 1000

    def __init__(self, options):
        super().__init__()
        self.session = c_int(0)

        self._generate_rfsg_calls()

        device = get_modinst_device("NI-RFSG", options.get("name", None))

        id_query = c_bool(False)
        reset = c_bool(True)
        self._niRFSG_init(device.device_name.encode(), id_query, reset,
                          byref(self.session))
        self._niRFSG_ConfigureGenerationMode(self.session,
                                             RFSGDevice.NIRFSG_VAL_CW)

    def __del__(self):
        if self.session.value > 0:
            self._niRFSG_close(self.session, 0)

    def _generate_rfsg_calls(self):
        """
        Generate needed Python function in class to do power measurements.
        For RFSGs the library has different names depending on the platform
        used. So platform is checked to select proper library name.
        """
        lib_name = "niRFSG_64.dll" if platform.architecture()[0] == '64bit' \
              else "niRFSG.dll"
        lib = CDLL(lib_name)
        # disable invalid name error to use function names as declared in C-DLL
        # pylint: disable=invalid-name
        self._niRFSG_init = _wrap(
            lib,
            "niRFSG_init",
            c_int,
            [c_char_p, c_bool, c_bool, POINTER(c_int)]
        )
        self._niRFSG_close = _wrap(
            lib,
            "niRFSG_close",
            c_int,
            [c_int, c_int]
        )

        self._niRFSG_SetAttributeViReal64 = _wrap(
            lib,
            "niRFSG_SetAttributeViReal64",
            c_int,
            [c_int, c_char_p, c_int, c_double]
        )

        self._niRFSG_GetAttributeViReal64 = _wrap(
            lib,
            "niRFSG_GetAttributeViReal64",
            c_int,
            [c_int, c_char_p, c_int, POINTER(c_double)]
        )

        self._niRFSG_ConfigureRF = _wrap(
            lib,
            "niRFSG_ConfigureRF",
            c_int,
            [c_int, c_double, c_double]
        )

        self._niRFSG_ConfigureGenerationMode = _wrap(
            lib,
            "niRFSG_ConfigureGenerationMode",
            c_int,
            [c_int, c_int]
        )

        self._niRFSG_Initiate = _wrap(
            lib,
            "niRFSG_Initiate",
            c_int,
            [c_int]
        )

        self._niRFSG_Abort = _wrap(
            lib,
            "niRFSG_Abort",
            c_int,
            [c_int]
        )

    def enable(self, enb=True):
        """
        Switch tone generation on of off
        """
        if enb:
            print("[SigGen] Starting tone generator.")
            self._niRFSG_Initiate(self.session)
        else:
            print("[SigGen] Stopping tone generator.")
            self._niRFSG_Abort(self.session)

    def set_frequency(self, freq):
        """
        Tune tone generator to new center frequency.
        """
        print("[SigGen] Tuning signal to {:.3f} MHz.".format(freq/1e6))
        self._niRFSG_ConfigureRF(self.session, freq, self.get_power())

    def _get_attribute(self, attr_id):
        """
        Wrapper to ease reading RFSG double attributes.
        """
        result = c_double(0)
        self._niRFSG_GetAttributeViReal64(self.session, b"", attr_id, byref(result))
        return result.value

    def _set_attribute(self, attr_id, value):
        """
        Wrapper to ease writing RFSG double attributes.
        """
        attr = c_double(value)
        self._niRFSG_SetAttributeViReal64(self.session, b"", attr_id, attr)

    def set_power(self, power_dbm):
        """
        Set power of output signal and return actual power.
        """
        self._set_attribute(RFSGDevice.NIRFSG_ATTR_POWER_LEVEL, power_dbm)
        return self.get_power()

    def get_power(self):
        """
        Get power of output signal.
        """
        return self._get_attribute(RFSGDevice.NIRFSG_ATTR_POWER_LEVEL)



#!/usr/bin/env python3
"""
Copyright 2020 Ettus Research, A National Instrument Brand

SPDX-License-Identifier: GPL-3.0-or-later

convert_cal_data: A utility to convert CSV-based calibration data into binary
cal data.
"""

import os
import glob
import argparse
import uhd

def csv_to_iq_cal(filename):
    """
    Opens the file "filename", parses the CSV data, and returns a
    uhd.usrp.cal.IQCal class.
    """
    def parse_metadata(handle):
        """
        Return name, serial, and timestamp, and verify version.
        """
        num_lines_md = 4
        metadata = {}
        for _ in range(num_lines_md):
            key, val = [x.strip() for x in handle.readline().split(",", 1)]
            if key == "version":
                if val != "0, 1":
                    raise RuntimeError("Unexpected version: " + val)
            elif key == "timestamp":
                metadata[key] = int(val)
            else:
                metadata[key] = val
        if handle.readline().strip() != "DATA STARTS HERE":
            raise RuntimeError("Unexpected header!")
        handle.readline() # Flush header
        return metadata
    with open(filename, 'r') as handle:
        metadata = parse_metadata(handle)
        print("Found metadata: ")
        for key, val in metadata.items():
            print("* {k}: {v}".format(k=key, v=val))
        iq_cal = uhd.usrp.cal.IQCal(
            metadata['name'], metadata['serial'], metadata['timestamp'])
        for line in handle.readlines():
            lo_freq, corr_real, corr_imag, supp_abs, supp_delta = \
                line.split(",")
            iq_cal.set_cal_coeff(
                float(lo_freq),
                complex(float(corr_real.strip()), float(corr_imag.strip())),
                float(supp_abs.strip()),
                float(supp_delta.strip()),
            )
    return iq_cal

def get_csv_location():
    """
    Returns the path to the location of the CSV files
    """
    return os.path.expandvars("$HOME/.uhd/cal")

def get_all_files(csv_location=None):
    """
    Returns a list of all CSV files in the standard location
    """
    csv_path = csv_location or get_csv_location()
    return glob.glob(os.path.join(csv_path, "*.csv"))

def parse_args():
    """ Parse args and return args object """
    parser = argparse.ArgumentParser(
        description="Convert CSV-Based cal data to binary format",
    )
    parser.add_argument(
        'files',
        help="List files to convert. If empty, will attempt to auto-detect "
             "files from the standard calibration location ($HOME/.uhd/cal)",
        nargs="*",
    )
    return parser.parse_args()


def main():
    """Go, go, go!"""
    args = parse_args()
    file_list = args.files or get_all_files()
    for filename in file_list:
        print("Converting {}...".format(filename))
        print("Identifying cal data type...")
        key = os.path.basename(filename).split('_cal')[0]
        if key not in ('rx_iq', 'tx_iq', 'tx_dc'):
            print("Unidentified cal data type: {}".format(key))
            continue
        iq_cal = csv_to_iq_cal(filename)
        uhd.usrp.cal.database.write_cal_data(
            key,
            iq_cal.get_serial(),
            iq_cal.serialize())
    return 0

if __name__ == "__main__":
    exit(main())



#
# Copyright 2020 Ettus Research, a National Instruments Brand
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Python UHD Module: Calibration sub-module
"""

# Disable PyLint because the entire libtypes modules is a list of renames. It is
# thus less redundant to do a wildcard import, even if generally discouraged.
# We could also paste the contents of libtypes.py into here, but by leaving it
# separate we avoid importing the lib module in this __init__ file.
# pylint: disable=wildcard-import
from .libtypes import *
# pylint: enable=wildcard-import

from .meas_device import get_meas_device
from .switch import get_switch
from .usrp_calibrator import get_usrp_calibrator



#
# Copyright 2020 Ettus Research, a National Instruments Brand
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Access to the property tree
"""

from . import libpyuhd as lib

PropertyTree = lib.property_tree



#
# Copyright 2020 Ettus Research, a National Instruments Brand
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
""" @package usrp
Python UHD module containing the MultiUSRP and other objects
"""

from .multi_usrp import MultiUSRP
# Disable PyLint because the entire libtypes modules is a list of renames. It is
# thus less redundant to do a wildcard import, even if generally discouraged.
# We could also paste the contents of libtypes.py into here, but by leaving it
# separate we avoid importing the lib module in this __init__ file.
# pylint: disable=wildcard-import
from .libtypes import *
# pylint: enable=wildcard-import

from . import cal



#
# Copyright 2017-2018 Ettus Research, a National Instruments Company
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
""" @package usrp
Python UHD module containing the MultiUSRP and other objects
"""

from .. import libpyuhd as lib

# Disable PyLint because we want to make it look like the following classes are
# defined in Python, but they're just renames of lib types. They therefore
# follow name conventions for Python classes, not for global constants.
# pylint: disable=invalid-name
SubdevSpecPair = lib.usrp.subdev_spec_pair
SubdevSpec = lib.usrp.subdev_spec
GPIOAtrReg = lib.usrp.gpio_atr_reg
GPIOAtrMode = lib.usrp.gpio_atr_mode
Unit = lib.usrp.unit
AuxDAC = lib.usrp.aux_dac
AuxADC = lib.usrp.aux_adc
SpecialProps = lib.usrp.special_props
Sampling = lib.usrp.sampling
FEConnection = lib.usrp.fe_connection
StreamArgs = lib.usrp.stream_args
RXStreamer = lib.usrp.rx_streamer
TXStreamer = lib.usrp.tx_streamer
# pylint: enable=invalid-name
